﻿using Microsoft.Extensions.Logging;
using Monorail.DrawningObjects;
using Monorail.Exceptions;
using Monorail.Generics;
using Monorail.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics.Metrics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Linq;
using Serilog;

namespace Monorail
{
    public partial class FormMonorailCollection : Form
    {
        private readonly MonorailGenericStorage _storage;

        public FormMonorailCollection()
        {
            InitializeComponent();
            _storage = new MonorailGenericStorage(pictureBoxCollection.Width,
                pictureBoxCollection.Height);
        }

        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;

            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i].Name);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
>= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }

        }

        private void addMonorailButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            FormMonorailConfig form = new();
            form.Show();
            Action<DrawningMonorail>? monorailDelegate = new((m) =>
            {
                try
                {
                    bool q = obj + m;
                    MessageBox.Show("Объект добавлен");
                    Log.Information($"Добавлен объект в коллекцию {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    m.ChangePictureBoxSize(pictureBoxCollection.Width, pictureBoxCollection.Height);
                    pictureBoxCollection.Image = obj.ShowMonorails();
                }
                catch (StorageOverflowException ex)
                {
                    Log.Warning($"Коллекция {listBoxStorages.SelectedItem.ToString() ?? string.Empty} переполнена");
                    MessageBox.Show(ex.Message);
                }
                catch(ArgumentException ex)
                {
                    Log.Warning($"Добавляемый объект уже существует в коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    MessageBox.Show("Добавляемый объект уже сущесвует в коллекции");
                }
            });
            form.AddEvent(monorailDelegate);
        }

        private void deleteMonorailButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            try
            {

                int pos = Convert.ToInt32(maskedTextBox.Text);
                var q = obj - pos;
                MessageBox.Show("Объект удален");
                Log.Information($"Удален объект из коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty} по номеру {pos}");
                pictureBoxCollection.Image = obj.ShowMonorails();
            }
            catch (MonorailNotFoundException ex)
            {
                Log.Warning($"Не получилось удалить объект из коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                MessageBox.Show(ex.Message);
            }
            catch (FormatException ex)
            {
                Log.Warning($"Было введено не число");
                MessageBox.Show("Введите число");
            }
        }

        private void updateCollectionButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowMonorails();

        }

        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
_storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowMonorails();
        }

        private void delStorageButton_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo,
MessageBoxIcon.Question) == DialogResult.Yes)
            {
                string name = listBoxStorages.SelectedItem.ToString()
                ?? string.Empty;
                _storage.DelSet(name);
                ReloadObjects();
                Log.Information($"Удален набор: {name}");
            }

        }

        private void addStorageButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(storageAddNameBox.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(storageAddNameBox.Text);
            ReloadObjects();
            Log.Information($"Добавлен набор: {storageAddNameBox.Text}");
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно",
                    "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    Log.Information($"Файл {saveFileDialog.FileName} успешно сохранен");
                }
                catch (Exception ex)
                {
                    Log.Warning("Не удалось сохранить");
                    MessageBox.Show($"Не сохранилось: {ex.Message}",
"Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }

        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно",
                    "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    Log.Information($"Файл {openFileDialog.FileName} успешно загружен");
                    foreach (var collection in _storage.Keys)
                    {
                        listBoxStorages.Items.Add(collection);
                    }
                    ReloadObjects();
                }
                catch (Exception ex)
                {
                    Log.Warning("Не удалось загрузить");
                    MessageBox.Show($"Не загрузилось: {ex.Message}",
"Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }
        }

        private void ButtonSortByType_Click(object sender, EventArgs e) => CompareMonorails(new MonorailCompareByType());

        private void ButtonSortByColor_Click(object sender, EventArgs e) => CompareMonorails(new MonorailCompareByColor());

        private void CompareMonorails(IComparer<DrawningMonorail?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            obj.Sort(comparer);
            pictureBoxCollection.Image = obj.ShowMonorails();
        }


    }
}
