﻿using Monorail.DrawningObjects;
using Monorail.Generics;
using Monorail.MovementStrategy;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Monorail.Generics
{
    internal class MonorailGenericStorage
    {
        readonly Dictionary<MonorailCollectionInfo, MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>> _monorailStorages;
        public List<MonorailCollectionInfo> Keys => _monorailStorages.Keys.ToList();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;
        private static readonly char _separatorForKeyValue = '|';
        private readonly char _separatorRecords = ';';
        private static readonly char _separatorForObject = ':';

        public MonorailGenericStorage(int pictureWidth, int pictureHeight)
        {
            _monorailStorages = new Dictionary<MonorailCollectionInfo,
                MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<MonorailCollectionInfo,
MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>> record in _monorailStorages)
            {
                StringBuilder records = new();
                foreach (DrawningMonorail? elem in record.Value.GetMonorails)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key.Name}{_separatorForKeyValue}{records}");
            }

            if (data.Length == 0)
            {
                throw new Exception("Невалиданя операция, нет данных для сохранения");
            }
            string toWrite = $"MonorailStorage{Environment.NewLine}{data}";
            var strs = toWrite.Split(new char[] { '\n', '\r' },
StringSplitOptions.RemoveEmptyEntries);

            using (StreamWriter sw = new(filename))
            {
                foreach (var str in strs)
                {
                    sw.WriteLine(str);
                }
            }
            return true;
        }

        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new IOException("Файл не найден");
            }
            using (StreamReader sr = new(filename))
            {
                string str = sr.ReadLine();
                var strs = str.Split(new char[] { '\n', '\r' },
StringSplitOptions.RemoveEmptyEntries);
                if (strs == null || strs.Length == 0)
                {
                    throw new IOException("Нет данных для загрузки");
                }
                if (!strs[0].StartsWith("MonorailStorage"))
                {
                    throw new IOException("Неверный формат данных");
                }
                _monorailStorages.Clear();
                do
                {
                    string[] record = str.Split(_separatorForKeyValue,
    StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        str = sr.ReadLine();
                        continue;
                    }
                    MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>
    collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords,
                    StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        DrawningMonorail? monorail =
                        elem?.CreateDrawningMonorail(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (monorail != null)
                        {
                            if (!(collection + monorail))
                            {
                                return false;
                            }
                        }
                    }
                    _monorailStorages.Add(new MonorailCollectionInfo(record[0], string.Empty), collection);

                    str = sr.ReadLine();
                } while (str != null);
            }



            return true;
        }

        public void AddSet(string name)
        {
            _monorailStorages.Add(new MonorailCollectionInfo(name, string.Empty), new MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>(_pictureWidth, _pictureHeight));
        }

        public void DelSet(string name)
        {
            if (!_monorailStorages.ContainsKey(new MonorailCollectionInfo(name, string.Empty)))
                return;
            _monorailStorages.Remove(new MonorailCollectionInfo(name, string.Empty));
        }

        public MonorailGenericCollection<DrawningMonorail, DrawningObjectMonorail>? this[string ind]
        {
            get
            {
                MonorailCollectionInfo indObj = new MonorailCollectionInfo(ind, string.Empty);
                if (_monorailStorages.ContainsKey(indObj))
                    return _monorailStorages[indObj];
                return null;
            }
        }
    }
}
