﻿using Monorail.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Monorail.Generics
{
    internal class SetGeneric<T>
        where T : class
    {
        private readonly List<T?> _places;

        public int Count => _places.Count;

        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }

        public void SortSet(IComparer<T?> comparer) => _places.Sort(comparer);

        public bool Insert(T monorail, IEqualityComparer<T>? equal = null)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            Insert(monorail, 0, equal);
            return true;

        }

        public bool Insert(T monorail, int position, IEqualityComparer<T>? equal = null)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            if (!(position >= 0 && position <= Count))
                return false;
            if(equal != null)
            {
                if (_places.Contains(monorail, equal))
                    throw new ArgumentException(nameof(monorail));
            }
            _places.Insert(position, monorail);
            return true;
        }
        public bool Remove(int position)
        {
            if (!(position >= 0 && position < Count))
                throw new MonorailNotFoundException(position);
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get {
                if (!(position >= 0 && position < Count))
                    return null;
                return _places[position];
            }
            set {
                if (!(position >= 0 && position < Count && _places.Count < _maxCount))
                    return;
                _places.Insert(position, value);
                return;
            }
        }

        public IEnumerable<T?> GetMonorails(int? maxMonorails = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxMonorails.HasValue && i == maxMonorails.Value)
                {
                    yield break;
                }
            }
        }

    }
}
